"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllErrors = exports.getAllInstructions = exports.getAllDefinedTypes = exports.getAllAccounts = exports.getAllPdas = exports.getAllPrograms = exports.rootNodeFromIdls = exports.rootNode = void 0;
const shared_1 = require("../shared");
const ProgramNode_1 = require("./ProgramNode");
function rootNode(program, additionalPrograms) {
    return {
        kind: 'rootNode',
        program,
        additionalPrograms: (additionalPrograms !== null && additionalPrograms !== void 0 ? additionalPrograms : []),
        standard: 'kinobi',
        // TODO: Replace with __VERSION__ variable when available.
        version: '0.19.0',
    };
}
exports.rootNode = rootNode;
function rootNodeFromIdls(program, additionalPrograms) {
    const resolveIdl = (idl) => typeof idl === 'string' ? (0, shared_1.readJson)(idl) : idl;
    const programNode = (0, ProgramNode_1.programNodeFromIdl)(resolveIdl(program));
    const additionalProgramNodes = additionalPrograms
        .map(resolveIdl)
        .map(ProgramNode_1.programNodeFromIdl);
    return rootNode(programNode, additionalProgramNodes);
}
exports.rootNodeFromIdls = rootNodeFromIdls;
function getAllPrograms(node) {
    return [node.program, ...node.additionalPrograms];
}
exports.getAllPrograms = getAllPrograms;
function getAllPdas(node) {
    return getAllPrograms(node).flatMap((program) => program.pdas);
}
exports.getAllPdas = getAllPdas;
function getAllAccounts(node) {
    return getAllPrograms(node).flatMap((program) => program.accounts);
}
exports.getAllAccounts = getAllAccounts;
function getAllDefinedTypes(node) {
    return getAllPrograms(node).flatMap((program) => program.definedTypes);
}
exports.getAllDefinedTypes = getAllDefinedTypes;
function getAllInstructions(node) {
    return getAllPrograms(node).flatMap((program) => program.instructions);
}
exports.getAllInstructions = getAllInstructions;
function getAllErrors(node) {
    return getAllPrograms(node).flatMap((program) => program.errors);
}
exports.getAllErrors = getAllErrors;
//# sourceMappingURL=RootNode.js.map