"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTypeNodeFromIdl = exports.TYPE_NODES = exports.REGISTERED_TYPE_NODE_KINDS = exports.STANDALONE_TYPE_NODE_KINDS = void 0;
const idl_1 = require("../../idl");
const DefinedTypeLinkNode_1 = require("../linkNodes/DefinedTypeLinkNode");
const ArrayTypeNode_1 = require("./ArrayTypeNode");
const BooleanTypeNode_1 = require("./BooleanTypeNode");
const BytesTypeNode_1 = require("./BytesTypeNode");
const EnumTypeNode_1 = require("./EnumTypeNode");
const MapTypeNode_1 = require("./MapTypeNode");
const NumberTypeNode_1 = require("./NumberTypeNode");
const OptionTypeNode_1 = require("./OptionTypeNode");
const PublicKeyTypeNode_1 = require("./PublicKeyTypeNode");
const SetTypeNode_1 = require("./SetTypeNode");
const SizePrefixTypeNode_1 = require("./SizePrefixTypeNode");
const StringTypeNode_1 = require("./StringTypeNode");
const StructTypeNode_1 = require("./StructTypeNode");
const TupleTypeNode_1 = require("./TupleTypeNode");
exports.STANDALONE_TYPE_NODE_KINDS = [
    'amountTypeNode',
    'arrayTypeNode',
    'booleanTypeNode',
    'bytesTypeNode',
    'dateTimeTypeNode',
    'enumTypeNode',
    'fixedSizeTypeNode',
    'hiddenPrefixTypeNode',
    'hiddenSuffixTypeNode',
    'mapTypeNode',
    'numberTypeNode',
    'optionTypeNode',
    'postOffsetTypeNode',
    'preOffsetTypeNode',
    'publicKeyTypeNode',
    'sentinelTypeNode',
    'setTypeNode',
    'sizePrefixTypeNode',
    'solAmountTypeNode',
    'stringTypeNode',
    'structTypeNode',
    'tupleTypeNode',
    'zeroableOptionTypeNode',
];
null;
exports.REGISTERED_TYPE_NODE_KINDS = [
    ...exports.STANDALONE_TYPE_NODE_KINDS,
    'structFieldTypeNode',
    'enumEmptyVariantTypeNode',
    'enumStructVariantTypeNode',
    'enumTupleVariantTypeNode',
];
null;
exports.TYPE_NODES = [
    ...exports.STANDALONE_TYPE_NODE_KINDS,
    'definedTypeLinkNode',
];
function isArrayOfSize(array, size) {
    return Array.isArray(array) && array.length === size;
}
const createTypeNodeFromIdl = (idlType) => {
    // Leaf.
    if (typeof idlType === 'string' && idl_1.IDL_TYPE_LEAVES.includes(idlType)) {
        if (idlType === 'bool')
            return (0, BooleanTypeNode_1.booleanTypeNode)();
        if (idlType === 'publicKey')
            return (0, PublicKeyTypeNode_1.publicKeyTypeNode)();
        if (idlType === 'string')
            return (0, SizePrefixTypeNode_1.sizePrefixTypeNode)((0, StringTypeNode_1.stringTypeNode)('utf8'), (0, NumberTypeNode_1.numberTypeNode)('u32'));
        if (idlType === 'bytes')
            return (0, SizePrefixTypeNode_1.sizePrefixTypeNode)((0, BytesTypeNode_1.bytesTypeNode)(), (0, NumberTypeNode_1.numberTypeNode)('u32'));
        return (0, NumberTypeNode_1.numberTypeNode)(idlType);
    }
    // Ensure eveything else is an object.
    if (typeof idlType !== 'object') {
        throw new Error(`TypeNode: Unsupported type ${JSON.stringify(idlType)}`);
    }
    // Array.
    if ('array' in idlType && isArrayOfSize(idlType.array, 2)) {
        return (0, ArrayTypeNode_1.arrayTypeNodeFromIdl)(idlType);
    }
    // Vec.
    if ('vec' in idlType) {
        return (0, ArrayTypeNode_1.arrayTypeNodeFromIdl)(idlType);
    }
    // Defined link.
    if ('defined' in idlType && typeof idlType.defined === 'string') {
        return (0, DefinedTypeLinkNode_1.definedTypeLinkNode)(idlType.defined);
    }
    // Enum.
    if ('kind' in idlType && idlType.kind === 'enum' && 'variants' in idlType) {
        return (0, EnumTypeNode_1.enumTypeNodeFromIdl)(idlType);
    }
    // Map.
    if (('hashMap' in idlType && isArrayOfSize(idlType.hashMap, 2)) ||
        ('bTreeMap' in idlType && isArrayOfSize(idlType.bTreeMap, 2))) {
        return (0, MapTypeNode_1.mapTypeNodeFromIdl)(idlType);
    }
    // Option.
    if ('option' in idlType || 'coption' in idlType) {
        return (0, OptionTypeNode_1.optionTypeNodeFromIdl)(idlType);
    }
    // Set.
    if ('hashSet' in idlType || 'bTreeSet' in idlType) {
        return (0, SetTypeNode_1.setTypeNodeFromIdl)(idlType);
    }
    // Struct.
    if ('kind' in idlType && idlType.kind === 'struct') {
        return (0, StructTypeNode_1.structTypeNodeFromIdl)(idlType);
    }
    // Tuple.
    if ('tuple' in idlType && Array.isArray(idlType.tuple)) {
        return (0, TupleTypeNode_1.tupleTypeNodeFromIdl)(idlType);
    }
    // Throw an error for unsupported types.
    throw new Error(`TypeNode: Unsupported type ${JSON.stringify(idlType)}`);
};
exports.createTypeNodeFromIdl = createTypeNodeFromIdl;
//# sourceMappingURL=TypeNode.js.map