"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionAccountMetaFragment = void 0;
const shared_1 = require("../../../shared");
const common_1 = require("./common");
function getInstructionAccountMetaFragment(instructionAccountNode) {
    const typeParam = `TAccount${(0, shared_1.pascalCase)(instructionAccountNode.name)}`;
    // Writable, signer.
    if (instructionAccountNode.isSigner === true &&
        instructionAccountNode.isWritable) {
        return (0, common_1.fragment)(`WritableSignerAccount<${typeParam}> & IAccountSignerMeta<${typeParam}>`)
            .addImports('solanaInstructions', ['WritableSignerAccount'])
            .addImports('solanaSigners', ['IAccountSignerMeta']);
    }
    // Readonly, signer.
    if (instructionAccountNode.isSigner === true) {
        return (0, common_1.fragment)(`ReadonlySignerAccount<${typeParam}> & IAccountSignerMeta<${typeParam}>`)
            .addImports('solanaInstructions', ['ReadonlySignerAccount'])
            .addImports('solanaSigners', ['IAccountSignerMeta']);
    }
    // Writable, non-signer or optional signer.
    if (instructionAccountNode.isWritable) {
        return (0, common_1.fragment)(`WritableAccount<${typeParam}>`).addImports('solanaInstructions', 'WritableAccount');
    }
    // Readonly, non-signer or optional signer.
    return (0, common_1.fragment)(`ReadonlyAccount<${typeParam}>`).addImports('solanaInstructions', 'ReadonlyAccount');
}
exports.getInstructionAccountMetaFragment = getInstructionAccountMetaFragment;
//# sourceMappingURL=instructionAccountMeta.js.map