"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionAccountTypeParamFragment = void 0;
const shared_1 = require("../../../shared");
const ImportMap_1 = require("../ImportMap");
const common_1 = require("./common");
function getInstructionAccountTypeParamFragment(scope) {
    const { instructionNode, instructionAccountNode, programNode, allowAccountMeta, linkables, } = scope;
    const typeParam = `TAccount${(0, shared_1.pascalCase)(instructionAccountNode.name)}`;
    const accountMeta = allowAccountMeta ? ' | IAccountMeta<string>' : '';
    const imports = new ImportMap_1.ImportMap();
    if (allowAccountMeta) {
        imports.add('solanaInstructions', 'IAccountMeta');
    }
    if (instructionNode.optionalAccountStrategy === 'omitted' &&
        instructionAccountNode.isOptional) {
        return (0, common_1.fragment)(`${typeParam} extends string${accountMeta} | undefined = undefined`, imports);
    }
    const defaultAddress = getDefaultAddress(instructionAccountNode.defaultValue, programNode.publicKey, linkables);
    return (0, common_1.fragment)(`${typeParam} extends string${accountMeta} = ${defaultAddress}`, imports);
}
exports.getInstructionAccountTypeParamFragment = getInstructionAccountTypeParamFragment;
function getDefaultAddress(defaultValue, programId, linkables) {
    switch (defaultValue === null || defaultValue === void 0 ? void 0 : defaultValue.kind) {
        case 'publicKeyValueNode':
            return `"${defaultValue.publicKey}"`;
        case 'programLinkNode':
            const programNode = linkables.get(defaultValue);
            return programNode ? `"${programNode.publicKey}"` : 'string';
        case 'programIdValueNode':
            return `"${programId}"`;
        default:
            return 'string';
    }
}
//# sourceMappingURL=instructionAccountTypeParam.js.map