"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPdaFunctionFragment = void 0;
const nodes_1 = require("../../../nodes");
const visitors_1 = require("../../../visitors");
const ImportMap_1 = require("../ImportMap");
const common_1 = require("./common");
function getPdaFunctionFragment(scope) {
    const { pdaNode, programNode, typeManifestVisitor, nameApi } = scope;
    // Seeds.
    const imports = new ImportMap_1.ImportMap();
    const seeds = pdaNode.seeds.map((seed) => {
        if ((0, nodes_1.isNode)(seed, 'variablePdaSeedNode')) {
            const seedManifest = (0, visitors_1.visit)(seed.type, typeManifestVisitor);
            imports.mergeWith(seedManifest.looseType, seedManifest.encoder);
            return { ...seed, typeManifest: seedManifest };
        }
        if ((0, nodes_1.isNode)(seed.value, 'programIdValueNode')) {
            imports.add('solanaAddresses', 'getAddressEncoder');
            return seed;
        }
        const seedManifest = (0, visitors_1.visit)(seed.type, typeManifestVisitor);
        imports.mergeWith(seedManifest.encoder);
        const seedValue = seed.value;
        const valueManifest = (0, visitors_1.visit)(seedValue, typeManifestVisitor).value;
        seedValue.render = valueManifest.render;
        imports.mergeWith(valueManifest.imports);
        return { ...seed, typeManifest: seedManifest };
    });
    const hasVariableSeeds = pdaNode.seeds.filter((0, nodes_1.isNodeFilter)('variablePdaSeedNode')).length > 0;
    return (0, common_1.fragmentFromTemplate)('pdaFunction.njk', {
        accountType: nameApi.accountType(pdaNode.name),
        pdaSeedsType: nameApi.pdaSeedsType(pdaNode.name),
        findPdaFunction: nameApi.pdaFindFunction(pdaNode.name),
        program: programNode,
        seeds,
        hasVariableSeeds,
    })
        .mergeImportsWith(imports)
        .addImports('solanaAddresses', [
        'Address',
        'getProgramDerivedAddress',
        'ProgramDerivedAddress',
    ]);
}
exports.getPdaFunctionFragment = getPdaFunctionFragment;
//# sourceMappingURL=pdaFunction.js.map