"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTypeManifestVisitor = void 0;
const nodes_1 = require("../../nodes");
const shared_1 = require("../../shared");
const visitors_1 = require("../../visitors");
const JavaScriptImportMap_1 = require("./JavaScriptImportMap");
function typeManifest() {
    return {
        isEnum: false,
        strictType: '',
        strictImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
        looseType: '',
        looseImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
        serializer: '',
        serializerImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
        value: '',
        valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
    };
}
function getTypeManifestVisitor(input) {
    var _a;
    const { linkables, nonScalarEnums, customAccountData, customInstructionData, } = input;
    let parentName = (_a = input.parentName) !== null && _a !== void 0 ? _a : null;
    let parentSize = null;
    return (0, shared_1.pipe)((0, visitors_1.staticVisitor)(() => ({
        isEnum: false,
        strictType: '',
        strictImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
        looseType: '',
        looseImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
        serializer: '',
        serializerImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
        value: '',
        valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
    }), [
        ...nodes_1.REGISTERED_TYPE_NODE_KINDS,
        ...nodes_1.REGISTERED_VALUE_NODE_KINDS,
        'definedTypeLinkNode',
        'definedTypeNode',
        'accountNode',
        'instructionNode',
    ]), (v) => (0, visitors_1.extendVisitor)(v, {
        visitAccount(account, { self }) {
            var _a;
            parentName = {
                strict: `${(0, shared_1.pascalCase)(account.name)}AccountData`,
                loose: `${(0, shared_1.pascalCase)(account.name)}AccountDataArgs`,
            };
            const link = (_a = customAccountData.get(account.name)) === null || _a === void 0 ? void 0 : _a.linkNode;
            const manifest = link ? (0, visitors_1.visit)(link, self) : (0, visitors_1.visit)(account.data, self);
            parentName = null;
            return manifest;
        },
        visitInstruction(instruction, { self }) {
            var _a;
            parentName = {
                strict: `${(0, shared_1.pascalCase)(instruction.name)}InstructionData`,
                loose: `${(0, shared_1.pascalCase)(instruction.name)}InstructionDataArgs`,
            };
            const link = (_a = customInstructionData.get(instruction.name)) === null || _a === void 0 ? void 0 : _a.linkNode;
            const struct = (0, nodes_1.structTypeNodeFromInstructionArgumentNodes)(instruction.arguments);
            const manifest = link ? (0, visitors_1.visit)(link, self) : (0, visitors_1.visit)(struct, self);
            parentName = null;
            return manifest;
        },
        visitDefinedType(definedType, { self }) {
            parentName = {
                strict: (0, shared_1.pascalCase)(definedType.name),
                loose: `${(0, shared_1.pascalCase)(definedType.name)}Args`,
            };
            const manifest = (0, visitors_1.visit)(definedType.type, self);
            parentName = null;
            return manifest;
        },
        visitArrayType(arrayType, { self }) {
            const childManifest = (0, visitors_1.visit)(arrayType.item, self);
            childManifest.serializerImports.add('umiSerializers', 'array');
            const sizeOption = getArrayLikeSizeOption(arrayType.count, childManifest, self);
            const options = sizeOption ? `, { ${sizeOption} }` : '';
            return {
                ...childManifest,
                strictType: `Array<${childManifest.strictType}>`,
                looseType: `Array<${childManifest.looseType}>`,
                serializer: `array(${childManifest.serializer + options})`,
            };
        },
        visitDefinedTypeLink(node) {
            var _a;
            const pascalCaseDefinedType = (0, shared_1.pascalCase)(node.name);
            const serializerName = `get${pascalCaseDefinedType}Serializer`;
            const importFrom = (_a = node.importFrom) !== null && _a !== void 0 ? _a : 'generatedTypes';
            return {
                isEnum: false,
                strictType: pascalCaseDefinedType,
                strictImports: new JavaScriptImportMap_1.JavaScriptImportMap().add(importFrom, pascalCaseDefinedType),
                looseType: `${pascalCaseDefinedType}Args`,
                looseImports: new JavaScriptImportMap_1.JavaScriptImportMap().add(importFrom, `${pascalCaseDefinedType}Args`),
                serializer: `${serializerName}()`,
                serializerImports: new JavaScriptImportMap_1.JavaScriptImportMap().add(importFrom, serializerName),
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitEnumType(enumType, { self }) {
            const strictImports = new JavaScriptImportMap_1.JavaScriptImportMap();
            const looseImports = new JavaScriptImportMap_1.JavaScriptImportMap();
            const serializerImports = new JavaScriptImportMap_1.JavaScriptImportMap();
            const variantNames = enumType.variants.map((variant) => (0, shared_1.pascalCase)(variant.name));
            const currentParentName = { ...parentName };
            parentName = null;
            const options = [];
            const enumSize = (0, nodes_1.resolveNestedTypeNode)(enumType.size);
            if (enumSize.format !== 'u8' || enumSize.endian !== 'le') {
                const sizeManifest = (0, visitors_1.visit)(enumType.size, self);
                strictImports.mergeWith(sizeManifest.strictImports);
                looseImports.mergeWith(sizeManifest.looseImports);
                serializerImports.mergeWith(sizeManifest.serializerImports);
                options.push(`size: ${sizeManifest.serializer}`);
            }
            if ((0, nodes_1.isScalarEnum)(enumType)) {
                if (currentParentName === null) {
                    throw new Error('Scalar enums cannot be inlined and must be introduced ' +
                        'via a defined type. Ensure you are not inlining a ' +
                        'defined type that is a scalar enum through a visitor.');
                }
                options.push(`description: '${currentParentName.strict}'`);
                const optionsAsString = options.length > 0 ? `, { ${options.join(', ')} }` : '';
                return {
                    isEnum: true,
                    strictType: `{ ${variantNames.join(', ')} }`,
                    strictImports,
                    looseType: `{ ${variantNames.join(', ')} }`,
                    looseImports,
                    serializer: `scalarEnum<${currentParentName.strict}>` +
                        `(${currentParentName.strict + optionsAsString})`,
                    serializerImports: serializerImports.add('umiSerializers', 'scalarEnum'),
                    value: '',
                    valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
                };
            }
            const variants = enumType.variants.map((variant) => {
                const variantName = (0, shared_1.pascalCase)(variant.name);
                parentName = currentParentName
                    ? {
                        strict: `GetDataEnumKindContent<${currentParentName.strict}, '${variantName}'>`,
                        loose: `GetDataEnumKindContent<${currentParentName.loose}, '${variantName}'>`,
                    }
                    : null;
                const variantManifest = (0, visitors_1.visit)(variant, self);
                parentName = null;
                return variantManifest;
            });
            const mergedManifest = mergeManifests(variants);
            mergedManifest.strictImports.mergeWith(strictImports);
            mergedManifest.looseImports.mergeWith(looseImports);
            mergedManifest.serializerImports.mergeWith(serializerImports);
            const variantSerializers = variants
                .map((variant) => variant.serializer)
                .join(', ');
            const serializerTypeParams = currentParentName
                ? currentParentName.strict
                : 'any';
            if (currentParentName === null || currentParentName === void 0 ? void 0 : currentParentName.strict) {
                options.push(`description: '${(0, shared_1.pascalCase)(currentParentName.strict)}'`);
            }
            const optionsAsString = options.length > 0 ? `, { ${options.join(', ')} }` : '';
            return {
                ...mergedManifest,
                strictType: variants
                    .map((variant) => variant.strictType)
                    .join(' | '),
                looseType: variants.map((variant) => variant.looseType).join(' | '),
                serializer: `dataEnum<${serializerTypeParams}>` +
                    `([${variantSerializers}]${optionsAsString})`,
                serializerImports: mergedManifest.serializerImports.add('umiSerializers', ['GetDataEnumKindContent', 'GetDataEnumKind', 'dataEnum']),
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitEnumEmptyVariantType(enumEmptyVariantType) {
            const name = (0, shared_1.pascalCase)(enumEmptyVariantType.name);
            const kindAttribute = `__kind: "${name}"`;
            return {
                isEnum: false,
                strictType: `{ ${kindAttribute} }`,
                strictImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
                looseType: `{ ${kindAttribute} }`,
                looseImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
                serializer: `['${name}', unit()]`,
                serializerImports: new JavaScriptImportMap_1.JavaScriptImportMap().add('umiSerializers', 'unit'),
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitEnumStructVariantType(enumStructVariantType, { self }) {
            const name = (0, shared_1.pascalCase)(enumStructVariantType.name);
            const kindAttribute = `__kind: "${name}"`;
            const type = (0, visitors_1.visit)(enumStructVariantType.struct, self);
            return {
                ...type,
                strictType: `{ ${kindAttribute},${type.strictType.slice(1, -1)}}`,
                looseType: `{ ${kindAttribute},${type.looseType.slice(1, -1)}}`,
                serializer: `['${name}', ${type.serializer}]`,
            };
        },
        visitEnumTupleVariantType(enumTupleVariantType, { self }) {
            const name = (0, shared_1.pascalCase)(enumTupleVariantType.name);
            const kindAttribute = `__kind: "${name}"`;
            const struct = (0, nodes_1.structTypeNode)([
                (0, nodes_1.structFieldTypeNode)({
                    name: 'fields',
                    type: enumTupleVariantType.tuple,
                }),
            ]);
            const type = (0, visitors_1.visit)(struct, self);
            return {
                ...type,
                strictType: `{ ${kindAttribute},${type.strictType.slice(1, -1)}}`,
                looseType: `{ ${kindAttribute},${type.looseType.slice(1, -1)}}`,
                serializer: `['${name}', ${type.serializer}]`,
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitMapType(mapType, { self }) {
            const key = (0, visitors_1.visit)(mapType.key, self);
            const value = (0, visitors_1.visit)(mapType.value, self);
            const mergedManifest = mergeManifests([key, value]);
            mergedManifest.serializerImports.add('umiSerializers', 'map');
            const sizeOption = getArrayLikeSizeOption(mapType.count, mergedManifest, self);
            const options = sizeOption ? `, { ${sizeOption} }` : '';
            return {
                ...mergedManifest,
                strictType: `Map<${key.strictType}, ${value.strictType}>`,
                looseType: `Map<${key.looseType}, ${value.looseType}>`,
                serializer: `map(${key.serializer}, ${value.serializer}${options})`,
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitOptionType(optionType, { self }) {
            const childManifest = (0, visitors_1.visit)(optionType.item, self);
            childManifest.strictImports.add('umi', 'Option');
            childManifest.looseImports.add('umi', 'OptionOrNullable');
            childManifest.serializerImports.add('umiSerializers', 'option');
            const options = [];
            // Prefix option.
            const optionPrefix = (0, nodes_1.resolveNestedTypeNode)(optionType.prefix);
            if (optionPrefix.format !== 'u8' || optionPrefix.endian !== 'le') {
                const prefixManifest = (0, visitors_1.visit)(optionType.prefix, self);
                childManifest.strictImports.mergeWith(prefixManifest.strictImports);
                childManifest.looseImports.mergeWith(prefixManifest.looseImports);
                childManifest.serializerImports.mergeWith(prefixManifest.serializerImports);
                options.push(`prefix: ${prefixManifest.serializer}`);
            }
            // Fixed option.
            if (optionType.fixed) {
                options.push(`fixed: true`);
            }
            const optionsAsString = options.length > 0 ? `, { ${options.join(', ')} }` : '';
            return {
                ...childManifest,
                strictType: `Option<${childManifest.strictType}>`,
                looseType: `OptionOrNullable<${childManifest.looseType}>`,
                serializer: `option(${childManifest.serializer}${optionsAsString})`,
            };
        },
        visitSetType(setType, { self }) {
            const childManifest = (0, visitors_1.visit)(setType.item, self);
            childManifest.serializerImports.add('umiSerializers', 'set');
            const sizeOption = getArrayLikeSizeOption(setType.count, childManifest, self);
            const options = sizeOption ? `, { ${sizeOption} }` : '';
            return {
                ...childManifest,
                strictType: `Set<${childManifest.strictType}>`,
                looseType: `Set<${childManifest.looseType}>`,
                serializer: `set(${childManifest.serializer + options})`,
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitStructType(structType, { self }) {
            const currentParentName = parentName;
            parentName = null;
            const fields = structType.fields.map((field) => (0, visitors_1.visit)(field, self));
            const mergedManifest = mergeManifests(fields);
            mergedManifest.serializerImports.add('umiSerializers', 'struct');
            const fieldSerializers = fields
                .map((field) => field.serializer)
                .join(', ');
            const structDescription = (currentParentName === null || currentParentName === void 0 ? void 0 : currentParentName.strict) &&
                !currentParentName.strict.match(/['"<>]/)
                ? `, { description: '${(0, shared_1.pascalCase)(currentParentName.strict)}' }`
                : '';
            const serializerTypeParams = currentParentName
                ? currentParentName.strict
                : 'any';
            const baseManifest = {
                ...mergedManifest,
                strictType: `{ ${fields
                    .map((field) => field.strictType)
                    .join('')} }`,
                looseType: `{ ${fields.map((field) => field.looseType).join('')} }`,
                serializer: `struct<${serializerTypeParams}>` +
                    `([${fieldSerializers}]${structDescription})`,
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
            const optionalFields = structType.fields.filter((f) => !!f.defaultValue);
            if (optionalFields.length === 0) {
                return baseManifest;
            }
            const defaultValues = optionalFields
                .map((f) => {
                const key = (0, shared_1.camelCase)(f.name);
                const defaultValue = f.defaultValue;
                const { value: renderedValue, valueImports } = (0, visitors_1.visit)(defaultValue, self);
                baseManifest.serializerImports.mergeWith(valueImports);
                if (f.defaultValueStrategy === 'omitted') {
                    return `${key}: ${renderedValue}`;
                }
                return `${key}: value.${key} ?? ${renderedValue}`;
            })
                .join(', ');
            const mapSerializerTypeParams = currentParentName
                ? `${currentParentName.loose}, any, ${currentParentName.strict}`
                : 'any, any, any';
            const mappedSerializer = `mapSerializer<${mapSerializerTypeParams}>(` +
                `${baseManifest.serializer}, ` +
                `(value) => ({ ...value, ${defaultValues} }) ` +
                `)`;
            baseManifest.serializerImports.add('umiSerializers', 'mapSerializer');
            return { ...baseManifest, serializer: mappedSerializer };
        },
        visitStructFieldType(structFieldType, { self }) {
            const name = (0, shared_1.camelCase)(structFieldType.name);
            const fieldChild = (0, visitors_1.visit)(structFieldType.type, self);
            const docblock = structFieldType.docs.length > 0
                ? `\n${(0, shared_1.jsDocblock)(structFieldType.docs)}`
                : '';
            const baseField = {
                ...fieldChild,
                strictType: `${docblock}${name}: ${fieldChild.strictType}; `,
                looseType: `${docblock}${name}: ${fieldChild.looseType}; `,
                serializer: `['${name}', ${fieldChild.serializer}]`,
            };
            if (!structFieldType.defaultValue) {
                return baseField;
            }
            if (structFieldType.defaultValueStrategy !== 'omitted') {
                return {
                    ...baseField,
                    looseType: `${docblock}${name}?: ${fieldChild.looseType}; `,
                };
            }
            return {
                ...baseField,
                looseType: '',
                looseImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitTupleType(tupleType, { self }) {
            const items = tupleType.items.map((item) => (0, visitors_1.visit)(item, self));
            const mergedManifest = mergeManifests(items);
            mergedManifest.serializerImports.add('umiSerializers', 'tuple');
            const itemSerializers = items
                .map((child) => child.serializer)
                .join(', ');
            return {
                ...mergedManifest,
                strictType: `[${items.map((item) => item.strictType).join(', ')}]`,
                looseType: `[${items.map((item) => item.looseType).join(', ')}]`,
                serializer: `tuple([${itemSerializers}])`,
                value: '',
            };
        },
        visitBooleanType(booleanType, { self }) {
            const looseImports = new JavaScriptImportMap_1.JavaScriptImportMap();
            const strictImports = new JavaScriptImportMap_1.JavaScriptImportMap();
            const serializerImports = new JavaScriptImportMap_1.JavaScriptImportMap().add('umiSerializers', 'bool');
            let sizeSerializer = '';
            const resolvedSize = (0, nodes_1.resolveNestedTypeNode)(booleanType.size);
            if (resolvedSize.format !== 'u8' || resolvedSize.endian !== 'le') {
                const size = (0, visitors_1.visit)(booleanType.size, self);
                looseImports.mergeWith(size.looseImports);
                strictImports.mergeWith(size.strictImports);
                serializerImports.mergeWith(size.serializerImports);
                sizeSerializer = `{ size: ${size.serializer} }`;
            }
            return {
                isEnum: false,
                strictType: 'boolean',
                looseType: 'boolean',
                serializer: `bool(${sizeSerializer})`,
                looseImports,
                strictImports,
                serializerImports,
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitBytesType(_bytesType, { self }) {
            const strictImports = new JavaScriptImportMap_1.JavaScriptImportMap();
            const looseImports = new JavaScriptImportMap_1.JavaScriptImportMap();
            const serializerImports = new JavaScriptImportMap_1.JavaScriptImportMap().add('umiSerializers', 'bytes');
            const options = [];
            // Size option.
            if (typeof parentSize === 'number') {
                options.push(`size: ${parentSize}`);
            }
            else if (parentSize) {
                const prefix = (0, visitors_1.visit)(parentSize, self);
                strictImports.mergeWith(prefix.strictImports);
                looseImports.mergeWith(prefix.looseImports);
                serializerImports.mergeWith(prefix.serializerImports);
                options.push(`size: ${prefix.serializer}`);
            }
            const optionsAsString = options.length > 0 ? `{ ${options.join(', ')} }` : '';
            return {
                isEnum: false,
                strictType: 'Uint8Array',
                strictImports,
                looseType: 'Uint8Array',
                looseImports,
                serializer: `bytes(${optionsAsString})`,
                serializerImports,
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitNumberType(numberType) {
            const isBigNumber = ['u64', 'u128', 'i64', 'i128'].includes(numberType.format);
            const serializerImports = new JavaScriptImportMap_1.JavaScriptImportMap().add('umiSerializers', numberType.format);
            let endianness = '';
            if (numberType.endian === 'be') {
                serializerImports.add('umiSerializers', 'Endian');
                endianness = '{ endian: Endian.Big }';
            }
            return {
                isEnum: false,
                strictType: isBigNumber ? 'bigint' : 'number',
                strictImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
                looseType: isBigNumber ? 'number | bigint' : 'number',
                looseImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
                serializer: `${numberType.format}(${endianness})`,
                serializerImports,
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitAmountType(amountType, { self }) {
            const numberManifest = (0, visitors_1.visit)(amountType.number, self);
            const resolvedNode = (0, nodes_1.resolveNestedTypeNode)(amountType.number);
            if (!(0, nodes_1.isUnsignedInteger)(resolvedNode)) {
                throw new Error(`Amount wrappers can only be applied to unsigned ` +
                    `integer types. Got type [${amountType.number.toString()}].`);
            }
            const { unit, decimals } = amountType;
            const idAndDecimals = `'${unit !== null && unit !== void 0 ? unit : 'Unknown'}', ${decimals}`;
            const isSolAmount = unit === 'SOL' && decimals === 9;
            const amountTypeString = isSolAmount
                ? 'SolAmount'
                : `Amount<${idAndDecimals}>`;
            const amountImport = isSolAmount ? 'SolAmount' : 'Amount';
            numberManifest.strictImports.add('umi', amountImport);
            numberManifest.looseImports.add('umi', amountImport);
            numberManifest.serializerImports.add('umi', 'mapAmountSerializer');
            return {
                ...numberManifest,
                strictType: amountTypeString,
                looseType: amountTypeString,
                serializer: `mapAmountSerializer(${numberManifest.serializer}, ${idAndDecimals})`,
            };
        },
        visitDateTimeType(dateTimeType, { self }) {
            const numberManifest = (0, visitors_1.visit)(dateTimeType.number, self);
            const dateTimeNumber = (0, nodes_1.resolveNestedTypeNode)(dateTimeType.number);
            if (!(0, nodes_1.isInteger)(dateTimeNumber)) {
                throw new Error(`DateTime wrappers can only be applied to integer ` +
                    `types. Got type [${dateTimeNumber.toString()}].`);
            }
            numberManifest.strictImports.add('umi', 'DateTime');
            numberManifest.looseImports.add('umi', 'DateTimeInput');
            numberManifest.serializerImports.add('umi', 'mapDateTimeSerializer');
            return {
                ...numberManifest,
                strictType: `DateTime`,
                looseType: `DateTimeInput`,
                serializer: `mapDateTimeSerializer(${numberManifest.serializer})`,
            };
        },
        visitSolAmountType(solAmountType, { self }) {
            const numberManifest = (0, visitors_1.visit)(solAmountType.number, self);
            const nestedNumber = (0, nodes_1.resolveNestedTypeNode)(solAmountType.number);
            if (!(0, nodes_1.isUnsignedInteger)(nestedNumber)) {
                throw new Error(`Amount wrappers can only be applied to unsigned ` +
                    `integer types. Got type [${nestedNumber.toString()}].`);
            }
            const idAndDecimals = `'SOL', 9`;
            numberManifest.strictImports.add('umi', 'SolAmount');
            numberManifest.looseImports.add('umi', 'SolAmount');
            numberManifest.serializerImports.add('umi', 'mapAmountSerializer');
            return {
                ...numberManifest,
                strictType: 'SolAmount',
                looseType: 'SolAmount',
                serializer: `mapAmountSerializer(${numberManifest.serializer}, ${idAndDecimals})`,
            };
        },
        visitPublicKeyType() {
            const imports = new JavaScriptImportMap_1.JavaScriptImportMap().add('umi', 'PublicKey');
            return {
                isEnum: false,
                strictType: 'PublicKey',
                strictImports: imports,
                looseType: 'PublicKey',
                looseImports: imports,
                serializer: `publicKeySerializer()`,
                serializerImports: new JavaScriptImportMap_1.JavaScriptImportMap()
                    .add('umiSerializers', 'publicKey')
                    .addAlias('umiSerializers', 'publicKey', 'publicKeySerializer'),
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitStringType(stringType, { self }) {
            const looseImports = new JavaScriptImportMap_1.JavaScriptImportMap();
            const strictImports = new JavaScriptImportMap_1.JavaScriptImportMap();
            const serializerImports = new JavaScriptImportMap_1.JavaScriptImportMap().add('umiSerializers', 'string');
            const options = [];
            // Encoding option.
            if (stringType.encoding !== 'utf8') {
                looseImports.add('umiSerializers', stringType.encoding);
                strictImports.add('umiSerializers', stringType.encoding);
                options.push(`encoding: ${stringType.encoding}`);
            }
            // Size option.
            if (!parentSize) {
                options.push(`size: 'variable'`);
            }
            else if (typeof parentSize === 'number') {
                options.push(`size: ${parentSize}`);
            }
            else if (parentSize.format !== 'u32' ||
                parentSize.endian !== 'le') {
                const prefix = (0, visitors_1.visit)(parentSize, self);
                looseImports.mergeWith(prefix.looseImports);
                strictImports.mergeWith(prefix.strictImports);
                serializerImports.mergeWith(prefix.serializerImports);
                options.push(`size: ${prefix.serializer}`);
            }
            const optionsAsString = options.length > 0 ? `{ ${options.join(', ')} }` : '';
            return {
                isEnum: false,
                strictType: 'string',
                strictImports,
                looseType: 'string',
                looseImports,
                serializer: `string(${optionsAsString})`,
                serializerImports,
                value: '',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap(),
            };
        },
        visitFixedSizeType(fixedSizeType, { self }) {
            parentSize = fixedSizeType.size;
            const manifest = (0, visitors_1.visit)(fixedSizeType.type, self);
            parentSize = null;
            return manifest;
        },
        visitSizePrefixType(sizePrefixType, { self }) {
            parentSize = (0, nodes_1.resolveNestedTypeNode)(sizePrefixType.prefix);
            const manifest = (0, visitors_1.visit)(sizePrefixType.type, self);
            parentSize = null;
            return manifest;
        },
        visitArrayValue(node, { self }) {
            const list = node.items.map((value) => (0, visitors_1.visit)(value, self));
            return {
                ...typeManifest(),
                value: `[${list.map((c) => c.value).join(', ')}]`,
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap().mergeWith(...list.map((c) => c.valueImports)),
            };
        },
        visitBooleanValue(node) {
            return {
                ...typeManifest(),
                value: JSON.stringify(node.boolean),
            };
        },
        visitBytesValue(node) {
            const bytes = (0, nodes_1.getBytesFromBytesValueNode)(node);
            return {
                ...typeManifest(),
                value: `new Uint8Array([${Array.from(bytes).join(', ')}])`,
            };
        },
        visitConstantValue(node, { self }) {
            if ((0, nodes_1.isNode)(node.type, 'bytesTypeNode') &&
                (0, nodes_1.isNode)(node.value, 'bytesValueNode')) {
                return (0, visitors_1.visit)(node.value, self);
            }
            const imports = new JavaScriptImportMap_1.JavaScriptImportMap();
            const value = (0, visitors_1.visit)(node.value, self);
            imports.mergeWith(value.valueImports);
            const type = (0, visitors_1.visit)(node.type, self);
            imports.mergeWith(type.serializerImports);
            return {
                ...typeManifest(),
                value: `${type.serializer}.serialize(${value.value})`,
                valueImports: imports,
            };
        },
        visitEnumValue(node, { self }) {
            var _a, _b;
            const imports = new JavaScriptImportMap_1.JavaScriptImportMap();
            const enumName = (0, shared_1.pascalCase)(node.enum.name);
            const variantName = (0, shared_1.pascalCase)(node.variant);
            const importFrom = (_a = node.enum.importFrom) !== null && _a !== void 0 ? _a : 'generatedTypes';
            const enumNode = (_b = linkables.get(node.enum)) === null || _b === void 0 ? void 0 : _b.type;
            const isScalar = enumNode && (0, nodes_1.isNode)(enumNode, 'enumTypeNode')
                ? (0, nodes_1.isScalarEnum)(enumNode)
                : !nonScalarEnums.includes(node.enum.name);
            if (!node.value && isScalar) {
                return {
                    ...typeManifest(),
                    value: `${enumName}.${variantName}`,
                    valueImports: imports.add(importFrom, enumName),
                };
            }
            const enumFn = (0, shared_1.camelCase)(node.enum.name);
            imports.add(importFrom, enumFn);
            if (!node.value) {
                return {
                    ...typeManifest(),
                    value: `${enumFn}('${variantName}')`,
                    valueImports: imports,
                };
            }
            const enumValue = (0, visitors_1.visit)(node.value, self);
            const fields = enumValue.value;
            imports.mergeWith(enumValue.valueImports);
            return {
                ...typeManifest(),
                value: `${enumFn}('${variantName}', ${fields})`,
                valueImports: imports,
            };
        },
        visitMapValue(node, { self }) {
            const map = node.entries.map((entry) => (0, visitors_1.visit)(entry, self));
            return {
                ...typeManifest(),
                value: `new Map([${map.map((c) => c.value).join(', ')}])`,
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap().mergeWith(...map.map((c) => c.valueImports)),
            };
        },
        visitMapEntryValue(node, { self }) {
            const mapKey = (0, visitors_1.visit)(node.key, self);
            const mapValue = (0, visitors_1.visit)(node.value, self);
            return {
                ...typeManifest(),
                imports: mapKey.valueImports.mergeWith(mapValue.valueImports),
                render: `[${mapKey.value}, ${mapValue.value}]`,
            };
        },
        visitNoneValue() {
            return {
                ...typeManifest(),
                value: 'none()',
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap().add('umi', 'none'),
            };
        },
        visitNumberValue(node) {
            return {
                ...typeManifest(),
                value: JSON.stringify(node.number),
            };
        },
        visitPublicKeyValue(node) {
            return {
                ...typeManifest(),
                value: `publicKey("${node.publicKey}")`,
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap().add('umi', 'publicKey'),
            };
        },
        visitSetValue(node, { self }) {
            const set = node.items.map((value) => (0, visitors_1.visit)(value, self));
            return {
                ...typeManifest(),
                value: `new Set([${set.map((c) => c.value).join(', ')}])`,
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap().mergeWith(...set.map((c) => c.valueImports)),
            };
        },
        visitSomeValue(node, { self }) {
            const child = (0, visitors_1.visit)(node.value, self);
            return {
                ...typeManifest(),
                value: `some(${child.value})`,
                valueImports: child.valueImports.add('umi', 'some'),
            };
        },
        visitStringValue(node) {
            return {
                ...typeManifest(),
                value: JSON.stringify(node.string),
            };
        },
        visitStructValue(node, { self }) {
            const struct = node.fields.map((field) => (0, visitors_1.visit)(field, self));
            return {
                ...typeManifest(),
                value: `{ ${struct.map((c) => c.value).join(', ')} }`,
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap().mergeWith(...struct.map((c) => c.valueImports)),
            };
        },
        visitStructFieldValue(node, { self }) {
            const structValue = (0, visitors_1.visit)(node.value, self);
            return {
                ...structValue,
                value: `${node.name}: ${structValue.value}`,
            };
        },
        visitTupleValue(node, { self }) {
            const list = node.items.map((value) => (0, visitors_1.visit)(value, self));
            return {
                ...typeManifest(),
                value: `[${list.map((c) => c.value).join(', ')}]`,
                valueImports: new JavaScriptImportMap_1.JavaScriptImportMap().mergeWith(...list.map((c) => c.valueImports)),
            };
        },
    }));
}
exports.getTypeManifestVisitor = getTypeManifestVisitor;
function mergeManifests(manifests) {
    return {
        strictImports: new JavaScriptImportMap_1.JavaScriptImportMap().mergeWith(...manifests.map((td) => td.strictImports)),
        looseImports: new JavaScriptImportMap_1.JavaScriptImportMap().mergeWith(...manifests.map((td) => td.looseImports)),
        serializerImports: new JavaScriptImportMap_1.JavaScriptImportMap().mergeWith(...manifests.map((td) => td.serializerImports)),
        valueImports: new JavaScriptImportMap_1.JavaScriptImportMap().mergeWith(...manifests.map((td) => td.valueImports)),
        isEnum: false,
    };
}
function getArrayLikeSizeOption(count, manifest, self) {
    if ((0, nodes_1.isNode)(count, 'fixedCountNode'))
        return `size: ${count.value}`;
    if ((0, nodes_1.isNode)(count, 'remainderCountNode'))
        return `size: 'remainder'`;
    const prefixManifest = (0, visitors_1.visit)(count.prefix, self);
    if (prefixManifest.serializer === 'u32()')
        return null;
    manifest.strictImports.mergeWith(prefixManifest.strictImports);
    manifest.looseImports.mergeWith(prefixManifest.looseImports);
    manifest.serializerImports.mergeWith(prefixManifest.serializerImports);
    return `size: ${prefixManifest.serializer}`;
}
//# sourceMappingURL=getTypeManifestVisitor.js.map