"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultVisitor = void 0;
const nodes_1 = require("../nodes");
const deduplicateIdenticalDefinedTypesVisitor_1 = require("./deduplicateIdenticalDefinedTypesVisitor");
const flattenInstructionDataArgumentsVisitor_1 = require("./flattenInstructionDataArgumentsVisitor");
const setAnchorDiscriminatorsVisitor_1 = require("./setAnchorDiscriminatorsVisitor");
const setFixedAccountSizesVisitor_1 = require("./setFixedAccountSizesVisitor");
const setInstructionAccountDefaultValuesVisitor_1 = require("./setInstructionAccountDefaultValuesVisitor");
const singleNodeVisitor_1 = require("./singleNodeVisitor");
const transformU8ArraysToBytesVisitor_1 = require("./transformU8ArraysToBytesVisitor");
const unwrapInstructionArgsDefinedTypesVisitor_1 = require("./unwrapInstructionArgsDefinedTypesVisitor");
const visitor_1 = require("./visitor");
function defaultVisitor() {
    return (0, singleNodeVisitor_1.rootNodeVisitor)((currentRoot) => {
        let root = currentRoot;
        const updateRoot = (visitor) => {
            const newRoot = (0, visitor_1.visit)(root, visitor);
            (0, nodes_1.assertIsNode)(newRoot, 'rootNode');
            root = newRoot;
        };
        // Defined types.
        updateRoot((0, deduplicateIdenticalDefinedTypesVisitor_1.deduplicateIdenticalDefinedTypesVisitor)());
        // Accounts.
        updateRoot((0, setAnchorDiscriminatorsVisitor_1.setAnchorDiscriminatorsVisitor)());
        updateRoot((0, setFixedAccountSizesVisitor_1.setFixedAccountSizesVisitor)());
        // Instructions.
        updateRoot((0, setInstructionAccountDefaultValuesVisitor_1.setInstructionAccountDefaultValuesVisitor)(setInstructionAccountDefaultValuesVisitor_1.DEFAULT_INSTRUCTION_ACCOUNT_DEFAULT_RULES));
        updateRoot((0, unwrapInstructionArgsDefinedTypesVisitor_1.unwrapInstructionArgsDefinedTypesVisitor)());
        updateRoot((0, flattenInstructionDataArgumentsVisitor_1.flattenInstructionDataArgumentsVisitor)());
        // Extras.
        updateRoot((0, transformU8ArraysToBytesVisitor_1.transformU8ArraysToBytesVisitor)());
        return root;
    });
}
exports.defaultVisitor = defaultVisitor;
//# sourceMappingURL=defaultVisitor.js.map