"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.flattenInstructionArguments = exports.flattenInstructionDataArgumentsVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const bottomUpTransformerVisitor_1 = require("./bottomUpTransformerVisitor");
function flattenInstructionDataArgumentsVisitor() {
    return (0, bottomUpTransformerVisitor_1.bottomUpTransformerVisitor)([
        {
            select: '[instructionNode]',
            transform: (instruction) => {
                (0, nodes_1.assertIsNode)(instruction, 'instructionNode');
                return (0, nodes_1.instructionNode)({
                    ...instruction,
                    arguments: (0, exports.flattenInstructionArguments)(instruction.arguments),
                });
            },
        },
    ]);
}
exports.flattenInstructionDataArgumentsVisitor = flattenInstructionDataArgumentsVisitor;
const flattenInstructionArguments = (nodes, options = '*') => {
    const camelCaseOptions = options === '*' ? options : options.map(shared_1.camelCase);
    const shouldInline = (node) => options === '*' || camelCaseOptions.includes((0, shared_1.camelCase)(node.name));
    const inlinedArguments = nodes.flatMap((node) => {
        if ((0, nodes_1.isNode)(node.type, 'structTypeNode') && shouldInline(node)) {
            return node.type.fields.map((field) => (0, nodes_1.instructionArgumentNode)({ ...field }));
        }
        return node;
    });
    const inlinedFieldsNames = inlinedArguments.map((arg) => arg.name);
    const duplicates = inlinedFieldsNames.filter((e, i, a) => a.indexOf(e) !== i);
    const uniqueDuplicates = [...new Set(duplicates)];
    const hasConflictingNames = uniqueDuplicates.length > 0;
    if (hasConflictingNames) {
        (0, shared_1.logWarn)(`Cound not flatten the attributes of a struct ` +
            `since this would cause the following attributes ` +
            `to conflict [${uniqueDuplicates.join(', ')}].` +
            'You may want to rename the conflicting attributes.');
    }
    return hasConflictingNames ? nodes : inlinedArguments;
};
exports.flattenInstructionArguments = flattenInstructionArguments;
//# sourceMappingURL=flattenInstructionDataArgumentsVisitor.js.map