"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefinedTypeHistogramVisitor = void 0;
const shared_1 = require("../shared");
const extendVisitor_1 = require("./extendVisitor");
const interceptVisitor_1 = require("./interceptVisitor");
const mergeVisitor_1 = require("./mergeVisitor");
const visitor_1 = require("./visitor");
function mergeHistograms(histograms) {
    const result = {};
    histograms.forEach((histogram) => {
        Object.keys(histogram).forEach((key) => {
            const mainCaseKey = key;
            if (result[mainCaseKey] === undefined) {
                result[mainCaseKey] = histogram[mainCaseKey];
            }
            else {
                result[mainCaseKey].total += histogram[mainCaseKey].total;
                result[mainCaseKey].inAccounts += histogram[mainCaseKey].inAccounts;
                result[mainCaseKey].inDefinedTypes +=
                    histogram[mainCaseKey].inDefinedTypes;
                result[mainCaseKey].inInstructionArgs +=
                    histogram[mainCaseKey].inInstructionArgs;
                result[mainCaseKey].directlyAsInstructionArgs +=
                    histogram[mainCaseKey].directlyAsInstructionArgs;
            }
        });
    });
    return result;
}
function getDefinedTypeHistogramVisitor() {
    let mode = null;
    let stackLevel = 0;
    return (0, shared_1.pipe)((0, mergeVisitor_1.mergeVisitor)(() => ({}), (_, histograms) => mergeHistograms(histograms)), (v) => (0, interceptVisitor_1.interceptVisitor)(v, (node, next) => {
        stackLevel += 1;
        const newNode = next(node);
        stackLevel -= 1;
        return newNode;
    }), (v) => (0, extendVisitor_1.extendVisitor)(v, {
        visitAccount(node, { self }) {
            mode = 'account';
            stackLevel = 0;
            const histogram = (0, visitor_1.visit)(node.data, self);
            mode = null;
            return histogram;
        },
        visitInstruction(node, { self }) {
            var _a, _b;
            mode = 'instruction';
            stackLevel = 0;
            const dataHistograms = node.arguments.map((arg) => (0, visitor_1.visit)(arg, self));
            const extraHistograms = ((_a = node.extraArguments) !== null && _a !== void 0 ? _a : []).map((arg) => (0, visitor_1.visit)(arg, self));
            mode = null;
            const subHistograms = ((_b = node.subInstructions) !== null && _b !== void 0 ? _b : []).map((ix) => (0, visitor_1.visit)(ix, self));
            return mergeHistograms([
                ...dataHistograms,
                ...extraHistograms,
                ...subHistograms,
            ]);
        },
        visitDefinedType(node, { self }) {
            mode = 'definedType';
            stackLevel = 0;
            const histogram = (0, visitor_1.visit)(node.type, self);
            mode = null;
            return histogram;
        },
        visitDefinedTypeLink(node) {
            if (node.importFrom) {
                return {};
            }
            return {
                [node.name]: {
                    total: 1,
                    inAccounts: Number(mode === 'account'),
                    inDefinedTypes: Number(mode === 'definedType'),
                    inInstructionArgs: Number(mode === 'instruction'),
                    directlyAsInstructionArgs: Number(mode === 'instruction' && stackLevel <= 1),
                },
            };
        },
    }));
}
exports.getDefinedTypeHistogramVisitor = getDefinedTypeHistogramVisitor;
//# sourceMappingURL=getDefinedTypeHistogramVisitor.js.map