"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.recordLinkablesVisitor = void 0;
const nodes_1 = require("../nodes");
const extendVisitor_1 = require("./extendVisitor");
function recordLinkablesVisitor(visitor, linkables) {
    const overriddenFunctions = {};
    if ('visitRoot' in visitor) {
        overriddenFunctions.visitRoot = function visitRoot(node, { next }) {
            linkables.recordAll([
                ...(0, nodes_1.getAllPrograms)(node),
                ...(0, nodes_1.getAllPdas)(node),
                ...(0, nodes_1.getAllAccounts)(node),
                ...(0, nodes_1.getAllDefinedTypes)(node),
            ]);
            return next(node);
        };
    }
    if ('visitProgram' in visitor) {
        overriddenFunctions.visitProgram = function visitProgram(node, { next }) {
            linkables.recordAll([
                node,
                ...node.pdas,
                ...node.accounts,
                ...node.definedTypes,
            ]);
            return next(node);
        };
    }
    if ('visitPda' in visitor) {
        overriddenFunctions.visitPda = function visitPda(node, { next }) {
            linkables.record(node);
            return next(node);
        };
    }
    if ('visitAccount' in visitor) {
        overriddenFunctions.visitAccount = function visitAccount(node, { next }) {
            linkables.record(node);
            return next(node);
        };
    }
    if ('visitDefinedType' in visitor) {
        overriddenFunctions.visitDefinedType = function visitDefinedType(node, { next }) {
            linkables.record(node);
            return next(node);
        };
    }
    return (0, extendVisitor_1.extendVisitor)(visitor, overriddenFunctions);
}
exports.recordLinkablesVisitor = recordLinkablesVisitor;
//# sourceMappingURL=recordLinkablesVisitor.js.map