"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setFixedAccountSizesVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const getByteSizeVisitor_1 = require("./getByteSizeVisitor");
const recordLinkablesVisitor_1 = require("./recordLinkablesVisitor");
const topDownTransformerVisitor_1 = require("./topDownTransformerVisitor");
const visitor_1 = require("./visitor");
function setFixedAccountSizesVisitor() {
    const linkables = new shared_1.LinkableDictionary();
    const byteSizeVisitor = (0, getByteSizeVisitor_1.getByteSizeVisitor)(linkables);
    const visitor = (0, topDownTransformerVisitor_1.topDownTransformerVisitor)([
        {
            select: (node) => (0, nodes_1.isNode)(node, 'accountNode') && node.size === undefined,
            transform: (node) => {
                (0, nodes_1.assertIsNode)(node, 'accountNode');
                const size = (0, visitor_1.visit)(node.data, byteSizeVisitor);
                if (size === null)
                    return node;
                return (0, nodes_1.accountNode)({ ...node, size });
            },
        },
    ], ['rootNode', 'programNode', 'accountNode']);
    return (0, recordLinkablesVisitor_1.recordLinkablesVisitor)(visitor, linkables);
}
exports.setFixedAccountSizesVisitor = setFixedAccountSizesVisitor;
//# sourceMappingURL=setFixedAccountSizesVisitor.js.map