"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.throwValidatorItemsVisitor = void 0;
const chalk_1 = __importDefault(require("chalk"));
const shared_1 = require("../shared");
const mapVisitor_1 = require("./mapVisitor");
function throwValidatorItemsVisitor(visitor, throwLevel = 'error') {
    // eslint-disable-next-line no-console
    return (0, mapVisitor_1.mapVisitor)(visitor, (validatorBag) => {
        const bag = validatorBag.orderByLevel();
        bag.log();
        const levelHistogram = bag.items.reduce((acc, item) => {
            var _a;
            acc[item.level] = ((_a = acc[item.level]) !== null && _a !== void 0 ? _a : 0) + 1;
            return acc;
        }, {});
        const maxLevel = Object.keys(levelHistogram)
            .map((level) => (0, shared_1.getLevelIndex)(level))
            .sort((a, b) => b - a)[0];
        if (maxLevel >= (0, shared_1.getLevelIndex)(throwLevel)) {
            const histogramString = Object.keys(levelHistogram)
                .map((level) => `${level}s: ${levelHistogram[level]}`)
                .join(', ');
            // eslint-disable-next-line no-console
            console.log(`${chalk_1.default.red(`Failed to validate the nodes.`)} ` +
                `${chalk_1.default.red.bold(`Found ${histogramString}.`)}\n`);
            process.exit(1);
        }
    });
}
exports.throwValidatorItemsVisitor = throwValidatorItemsVisitor;
//# sourceMappingURL=throwValidatorItemsVisitor.js.map