"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformDefinedTypesIntoAccountsVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const extendVisitor_1 = require("./extendVisitor");
const nonNullableIdentityVisitor_1 = require("./nonNullableIdentityVisitor");
function transformDefinedTypesIntoAccountsVisitor(definedTypes) {
    return (0, shared_1.pipe)((0, nonNullableIdentityVisitor_1.nonNullableIdentityVisitor)(['rootNode', 'programNode']), (v) => (0, extendVisitor_1.extendVisitor)(v, {
        visitProgram(program) {
            const typesToExtract = program.definedTypes.filter((node) => definedTypes.includes(node.name));
            const newDefinedTypes = program.definedTypes.filter((node) => !definedTypes.includes(node.name));
            const newAccounts = typesToExtract.map((node) => {
                (0, nodes_1.assertIsNode)(node.type, 'structTypeNode');
                return (0, nodes_1.accountNode)({
                    ...node,
                    data: node.type,
                    size: undefined,
                    discriminators: [],
                });
            });
            return (0, nodes_1.programNode)({
                ...program,
                accounts: [...program.accounts, ...newAccounts],
                definedTypes: newDefinedTypes,
            });
        },
    }));
}
exports.transformDefinedTypesIntoAccountsVisitor = transformDefinedTypesIntoAccountsVisitor;
//# sourceMappingURL=transformDefinedTypesIntoAccountsVisitor.js.map