"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unwrapDefinedTypesVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const extendVisitor_1 = require("./extendVisitor");
const nonNullableIdentityVisitor_1 = require("./nonNullableIdentityVisitor");
const recordLinkablesVisitor_1 = require("./recordLinkablesVisitor");
const visitor_1 = require("./visitor");
function unwrapDefinedTypesVisitor(typesToInline = '*') {
    const linkables = new shared_1.LinkableDictionary();
    const typesToInlineMainCased = typesToInline === '*' ? '*' : typesToInline.map(shared_1.mainCase);
    const shouldInline = (definedType) => typesToInlineMainCased === '*' ||
        typesToInlineMainCased.includes(definedType);
    return (0, shared_1.pipe)((0, nonNullableIdentityVisitor_1.nonNullableIdentityVisitor)(), (v) => (0, recordLinkablesVisitor_1.recordLinkablesVisitor)(v, linkables), (v) => (0, extendVisitor_1.extendVisitor)(v, {
        visitProgram(program, { self }) {
            return (0, nodes_1.programNode)({
                ...program,
                accounts: program.accounts
                    .map((account) => (0, visitor_1.visit)(account, self))
                    .filter((0, nodes_1.assertIsNodeFilter)('accountNode')),
                instructions: program.instructions
                    .map((instruction) => (0, visitor_1.visit)(instruction, self))
                    .filter((0, nodes_1.assertIsNodeFilter)('instructionNode')),
                definedTypes: program.definedTypes
                    .filter((definedType) => !shouldInline(definedType.name))
                    .map((type) => (0, visitor_1.visit)(type, self))
                    .filter((0, nodes_1.assertIsNodeFilter)('definedTypeNode')),
            });
        },
        visitDefinedTypeLink(linkType, { self }) {
            if (!shouldInline(linkType.name) || linkType.importFrom) {
                return linkType;
            }
            const definedType = linkables.get(linkType);
            if (definedType === undefined) {
                throw new Error(`Trying to inline missing defined type [${linkType.name}]. ` +
                    `Ensure this visitor starts from the root node to access all defined types.`);
            }
            return (0, visitor_1.visit)(definedType.type, self);
        },
    }));
}
exports.unwrapDefinedTypesVisitor = unwrapDefinedTypesVisitor;
//# sourceMappingURL=unwrapDefinedTypesVisitor.js.map