"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unwrapTupleEnumWithSingleStructVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const bottomUpTransformerVisitor_1 = require("./bottomUpTransformerVisitor");
const getDefinedTypeHistogramVisitor_1 = require("./getDefinedTypeHistogramVisitor");
const singleNodeVisitor_1 = require("./singleNodeVisitor");
const unwrapDefinedTypesVisitor_1 = require("./unwrapDefinedTypesVisitor");
const visitor_1 = require("./visitor");
function unwrapTupleEnumWithSingleStructVisitor(enumsOrVariantsToUnwrap = '*') {
    const selectorFunctions = enumsOrVariantsToUnwrap === '*'
        ? [() => true]
        : enumsOrVariantsToUnwrap.map((selector) => (0, shared_1.getNodeSelectorFunction)(selector));
    const shouldUnwrap = (node, stack) => selectorFunctions.some((selector) => selector(node, stack));
    return (0, singleNodeVisitor_1.rootNodeVisitor)((root) => {
        const typesToPotentiallyUnwrap = [];
        const definedTypes = new Map((0, nodes_1.getAllDefinedTypes)(root).map((definedType) => [
            definedType.name,
            definedType,
        ]));
        let newRoot = (0, visitor_1.visit)(root, (0, bottomUpTransformerVisitor_1.bottomUpTransformerVisitor)([
            {
                select: '[enumTupleVariantTypeNode]',
                transform: (node, stack) => {
                    (0, nodes_1.assertIsNode)(node, 'enumTupleVariantTypeNode');
                    if (!shouldUnwrap(node, stack))
                        return node;
                    const tupleNode = (0, nodes_1.resolveNestedTypeNode)(node.tuple);
                    if (tupleNode.items.length !== 1)
                        return node;
                    let item = tupleNode.items[0];
                    if ((0, nodes_1.isNode)(item, 'definedTypeLinkNode')) {
                        if (item.importFrom)
                            return node;
                        const definedType = definedTypes.get(item.name);
                        if (!definedType)
                            return node;
                        if (!(0, nodes_1.isNode)(definedType.type, 'structTypeNode'))
                            return node;
                        typesToPotentiallyUnwrap.push(item.name);
                        item = definedType.type;
                    }
                    if (!(0, nodes_1.isNode)(item, 'structTypeNode'))
                        return node;
                    const nestedStruct = (0, nodes_1.transformNestedTypeNode)(node.tuple, () => item);
                    return (0, nodes_1.enumStructVariantTypeNode)(node.name, nestedStruct);
                },
            },
        ]));
        (0, nodes_1.assertIsNode)(newRoot, 'rootNode');
        const histogram = (0, visitor_1.visit)(newRoot, (0, getDefinedTypeHistogramVisitor_1.getDefinedTypeHistogramVisitor)());
        const typesToUnwrap = typesToPotentiallyUnwrap.filter((type) => !histogram[type] ||
            histogram[type].total === 0);
        newRoot = (0, visitor_1.visit)(newRoot, (0, unwrapDefinedTypesVisitor_1.unwrapDefinedTypesVisitor)(typesToUnwrap));
        (0, nodes_1.assertIsNode)(newRoot, 'rootNode');
        return newRoot;
    });
}
exports.unwrapTupleEnumWithSingleStructVisitor = unwrapTupleEnumWithSingleStructVisitor;
//# sourceMappingURL=unwrapTupleEnumWithSingleStructVisitor.js.map