"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unwrapTypeDefinedLinksVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const bottomUpTransformerVisitor_1 = require("./bottomUpTransformerVisitor");
const recordLinkablesVisitor_1 = require("./recordLinkablesVisitor");
function unwrapTypeDefinedLinksVisitor(definedLinksType) {
    const linkables = new shared_1.LinkableDictionary();
    const transformers = definedLinksType.map((selector) => ({
        select: ['[definedTypeLinkNode]', selector],
        transform: (node) => {
            (0, nodes_1.assertIsNode)(node, 'definedTypeLinkNode');
            if (node.importFrom)
                return node;
            const definedType = linkables.get(node);
            if (definedType === undefined) {
                throw new Error(`Trying to inline missing defined type [${node.name}]. ` +
                    `Ensure this visitor starts from the root node to access all defined types.`);
            }
            return definedType.type;
        },
    }));
    return (0, shared_1.pipe)((0, bottomUpTransformerVisitor_1.bottomUpTransformerVisitor)(transformers), (v) => (0, recordLinkablesVisitor_1.recordLinkablesVisitor)(v, linkables));
}
exports.unwrapTypeDefinedLinksVisitor = unwrapTypeDefinedLinksVisitor;
//# sourceMappingURL=unwrapTypeDefinedLinksVisitor.js.map