"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateProgramsVisitor = void 0;
const nodes_1 = require("../nodes");
const shared_1 = require("../shared");
const bottomUpTransformerVisitor_1 = require("./bottomUpTransformerVisitor");
function updateProgramsVisitor(map) {
    return (0, bottomUpTransformerVisitor_1.bottomUpTransformerVisitor)(Object.entries(map).flatMap(([name, updates]) => {
        const newName = typeof updates === 'object' && 'name' in updates && updates.name
            ? (0, shared_1.mainCase)(updates.name)
            : undefined;
        const transformers = [
            {
                select: `[programNode]${name}`,
                transform: (node) => {
                    (0, nodes_1.assertIsNode)(node, 'programNode');
                    if ('delete' in updates)
                        return null;
                    return (0, nodes_1.programNode)({ ...node, ...updates });
                },
            },
        ];
        if (newName) {
            transformers.push({
                select: `[programLinkNode]${name}`,
                transform: (node) => {
                    (0, nodes_1.assertIsNode)(node, 'programLinkNode');
                    if (node.importFrom)
                        return node;
                    return (0, nodes_1.programLinkNode)(newName);
                },
            });
        }
        return transformers;
    }));
}
exports.updateProgramsVisitor = updateProgramsVisitor;
//# sourceMappingURL=updateProgramsVisitor.js.map