import type { IdlInstruction } from '../idl';
import { MainCaseString } from '../shared';
import { InstructionAccountNode } from './InstructionAccountNode';
import { InstructionArgumentNode } from './InstructionArgumentNode';
import { InstructionByteDeltaNode } from './InstructionByteDeltaNode';
import { InstructionRemainingAccountsNode } from './InstructionRemainingAccountsNode';
import { ProgramNode } from './ProgramNode';
import { RootNode } from './RootNode';
import { DiscriminatorNode } from './discriminatorNodes';
type SubInstructionNode = InstructionNode;
export interface InstructionNode<TAccounts extends InstructionAccountNode[] = InstructionAccountNode[], TArguments extends InstructionArgumentNode[] = InstructionArgumentNode[], TExtraArguments extends InstructionArgumentNode[] | undefined = InstructionArgumentNode[] | undefined, TRemainingAccounts extends InstructionRemainingAccountsNode[] | undefined = InstructionRemainingAccountsNode[] | undefined, TByteDeltas extends InstructionByteDeltaNode[] | undefined = InstructionByteDeltaNode[] | undefined, TDiscriminators extends DiscriminatorNode[] | undefined = DiscriminatorNode[] | undefined, TSubInstructions extends SubInstructionNode[] | undefined = SubInstructionNode[] | undefined> {
    readonly kind: 'instructionNode';
    readonly accounts: TAccounts;
    readonly arguments: TArguments;
    readonly extraArguments?: TExtraArguments;
    readonly remainingAccounts?: TRemainingAccounts;
    readonly byteDeltas?: TByteDeltas;
    readonly discriminators?: TDiscriminators;
    readonly subInstructions?: TSubInstructions;
    readonly name: MainCaseString;
    readonly idlName: string;
    readonly docs: string[];
    readonly optionalAccountStrategy: 'omitted' | 'programId';
}
export type InstructionNodeInput<TAccounts extends InstructionAccountNode[] = InstructionAccountNode[], TArguments extends InstructionArgumentNode[] = InstructionArgumentNode[], TExtraArguments extends InstructionArgumentNode[] | undefined = InstructionArgumentNode[] | undefined, TRemainingAccounts extends InstructionRemainingAccountsNode[] | undefined = InstructionRemainingAccountsNode[] | undefined, TByteDeltas extends InstructionByteDeltaNode[] | undefined = InstructionByteDeltaNode[] | undefined, TDiscriminators extends DiscriminatorNode[] | undefined = DiscriminatorNode[] | undefined, TSubInstructions extends SubInstructionNode[] | undefined = SubInstructionNode[] | undefined> = Omit<Partial<InstructionNode<TAccounts, TArguments, TExtraArguments, TRemainingAccounts, TByteDeltas, TDiscriminators, TSubInstructions>>, 'kind' | 'name'> & {
    readonly name: string;
};
export declare function instructionNode<const TAccounts extends InstructionAccountNode[] = [], const TArguments extends InstructionArgumentNode[] = [], const TExtraArguments extends InstructionArgumentNode[] | undefined = undefined, const TRemainingAccounts extends InstructionRemainingAccountsNode[] | undefined = undefined, const TByteDeltas extends InstructionByteDeltaNode[] | undefined = undefined, const TDiscriminators extends DiscriminatorNode[] | undefined = undefined, const TSubInstructions extends SubInstructionNode[] | undefined = undefined>(input: InstructionNodeInput<TAccounts, TArguments, TExtraArguments, TRemainingAccounts, TByteDeltas, TDiscriminators, TSubInstructions>): InstructionNode<TAccounts, TArguments, TExtraArguments, TRemainingAccounts, TByteDeltas, TDiscriminators, TSubInstructions>;
export declare function instructionNodeFromIdl(idl: Partial<IdlInstruction>): InstructionNode;
export declare function getAllInstructionArguments(node: InstructionNode): InstructionArgumentNode[];
export declare function getAllInstructionsWithSubs(node: ProgramNode | RootNode | InstructionNode, config?: {
    leavesOnly?: boolean;
    subInstructionsFirst?: boolean;
}): InstructionNode[];
export {};
