import type { Idl } from '../idl';
import { MainCaseString, PartialExcept } from '../shared';
import { AccountNode } from './AccountNode';
import { DefinedTypeNode } from './DefinedTypeNode';
import { ErrorNode } from './ErrorNode';
import { InstructionNode } from './InstructionNode';
import { PdaNode } from './PdaNode';
export interface ProgramNode<TPdas extends PdaNode[] = PdaNode[], TAccounts extends AccountNode[] = AccountNode[], TInstructions extends InstructionNode[] = InstructionNode[], TDefinedTypes extends DefinedTypeNode[] = DefinedTypeNode[], TErrors extends ErrorNode[] = ErrorNode[]> {
    readonly kind: 'programNode';
    readonly pdas: TPdas;
    readonly accounts: TAccounts;
    readonly instructions: TInstructions;
    readonly definedTypes: TDefinedTypes;
    readonly errors: TErrors;
    readonly name: MainCaseString;
    readonly prefix: MainCaseString;
    readonly publicKey: string;
    readonly version: string;
    readonly origin?: 'shank' | 'anchor';
}
export type ProgramNodeInput<TPdas extends PdaNode[] = PdaNode[], TAccounts extends AccountNode[] = AccountNode[], TInstructions extends InstructionNode[] = InstructionNode[], TDefinedTypes extends DefinedTypeNode[] = DefinedTypeNode[], TErrors extends ErrorNode[] = ErrorNode[]> = Omit<PartialExcept<ProgramNode<TPdas, TAccounts, TInstructions, TDefinedTypes, TErrors>, 'publicKey'>, 'kind' | 'name' | 'prefix'> & {
    readonly name: string;
    readonly prefix?: string;
};
export declare function programNode<const TPdas extends PdaNode[] = [], const TAccounts extends AccountNode[] = [], const TInstructions extends InstructionNode[] = [], const TDefinedTypes extends DefinedTypeNode[] = [], const TErrors extends ErrorNode[] = []>(input: ProgramNodeInput<TPdas, TAccounts, TInstructions, TDefinedTypes, TErrors>): ProgramNode<TPdas, TAccounts, TInstructions, TDefinedTypes, TErrors>;
export declare function programNodeFromIdl(idl: Partial<Idl>): ProgramNode;
