import type { Idl } from '../idl';
import type { AccountNode } from './AccountNode';
import type { DefinedTypeNode } from './DefinedTypeNode';
import type { ErrorNode } from './ErrorNode';
import type { InstructionNode } from './InstructionNode';
import { PdaNode } from './PdaNode';
import { ProgramNode } from './ProgramNode';
export type IdlInput = string | Partial<Idl>;
export type KinobiVersion = `${number}.${number}.${number}`;
export interface RootNode<TProgram extends ProgramNode = ProgramNode, TAdditionalPrograms extends ProgramNode[] = ProgramNode[]> {
    readonly kind: 'rootNode';
    readonly program: TProgram;
    readonly additionalPrograms: TAdditionalPrograms;
    readonly standard: 'kinobi';
    readonly version: KinobiVersion;
}
export declare function rootNode<TProgram extends ProgramNode, const TAdditionalPrograms extends ProgramNode[] = []>(program: TProgram, additionalPrograms?: TAdditionalPrograms): RootNode<TProgram, TAdditionalPrograms>;
export declare function rootNodeFromIdls(program: IdlInput, additionalPrograms: IdlInput[]): RootNode;
export declare function getAllPrograms(node: RootNode): ProgramNode[];
export declare function getAllPdas(node: RootNode): PdaNode[];
export declare function getAllAccounts(node: RootNode): AccountNode[];
export declare function getAllDefinedTypes(node: RootNode): DefinedTypeNode[];
export declare function getAllInstructions(node: RootNode): InstructionNode[];
export declare function getAllErrors(node: RootNode): ErrorNode[];
